local M = {}

local debug = require("../debug")

M.type = "auxilliary"
M.relevantDevice = nil
M.defaultOrder = 900

local battery = nil
local hybrid = nil

local function init(jbeamData)
    M.idlePowerDraw = jbeamData.idlePowerDraw or 3 -- kW
    M.efficiency = jbeamData.efficiency or 0.91 -- %
    M.maxRegenPower = jbeamData.maxRegenPower or 60 -- kW
end

local function initSecondStage()
    battery = energyStorage.getStorage("mainBattery")
end

local function updateGFX(dt)
    if not battery or not hybrid then
        return
    end

    -- power values are in kW; dt is seconds
    local kJMaxRegen = M.maxRegenPower * dt
    local kJDrivetrain = hybrid.powerConsumption * dt
    local kJIdle = M.idlePowerDraw * dt
    local kJ = kJDrivetrain + kJIdle

    if kJ > 0 then
        -- Consuming power; lack of efficiency increases power drawn from battery (heat, etc)

        kJ = kJ / M.efficiency
    else
        -- Generating power; lack of efficiency decreases power stored in battery (again, heat)

        kJ = math.min(kJ, kJMaxRegen)
        kJ = kJ * M.efficiency
        
        local batteryLevel = battery.storedEnergy / battery.initialStoredEnergy
        local distToFull = clamp((0.9 - batteryLevel) / 0.1, 0.0, 1.0)
        
        kJ = kJ * distToFull -- Limit regen near full
    end

    -- convert kJ to J, convert from battery
    battery.storedEnergy = math.max(battery.storedEnergy - (kJ * 1000), 0.0)

    debug.dText(3, "kW: " .. (kJ / dt))
    debug.dText(4, "dt: " .. dt)
    debug.dText(5, "Battery: " .. battery.storedEnergy * 100 / battery.initialStoredEnergy .. "%")
end

local function registerHybridSystem(hybridController)
    hybrid = hybridController
end

local function getChargeLevel()
    return battery.storedEnergy / battery.initialStoredEnergy
end

M.init = init
M.initSecondStage = initSecondStage
M.updateGFX = updateGFX
M.registerHybridSystem = registerHybridSystem
M.getChargeLevel = getChargeLevel

return M
